from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject, IncrementalChangeElement, ItemRevision
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ICFilterCriteria(TcBaseObj):
    """
    ICFilterCriteria structure captures information about filtering criteria of incremental change contexts.
    
    :var owningUser: The owning user of the incremental change revision object(s) to search for.
    :var owningGroup: The owning group of the incremental change revision object(s) to search for.
    :var statuses: The details for status names and their start date.
    :var intents: A list of intents associated with Incremental Change contexts that should included in the search.
    These strings must be valid Intent names in Teamcenter.
    :var includedICRevs: A list of ItemRevision objects representing preselected incremental change contexts to be used
    for seaching incremental change elements. This list is merged with the results of the other search parameters.
    :var excludedICRevs: A list of ItemRevision objects representing preselected incremental change contexts to be
    excluded from the search
    """
    owningUser: str = ''
    owningGroup: str = ''
    statuses: StatusData = None
    intents: List[str] = ()
    includedICRevs: List[ItemRevision] = ()
    excludedICRevs: List[ItemRevision] = ()


@dataclass
class ICSearchInfo(TcBaseObj):
    """
    ICSearchInfo structure contains information for scoping the Incremental Change contexts to be intersected while
    searching for Incremental Change Elements tracking the changes in the structure. 
    
    :var isUnitData: If true unit data is being provided as opposed to date information.
    :var unitData: The details for searching changes based on unit information ( start Unit, end Unit and the end Item
    for the Unit effectivity).
    :var dateEffectivityData: The details for searching changes based on date (start Date and end Date).
    :var icFilterCriteria: The details for searching Incremental change contexts. It is only relevant for Incremental
    Change
    """
    isUnitData: bool = False
    unitData: EndItemUnitData = None
    dateEffectivityData: DateEffectivityData = None
    icFilterCriteria: ICFilterCriteria = None


@dataclass
class LineIceInfo(TcBaseObj):
    """
    LineIceInfo structure details the changes in Incremental Change context for each BOMLine. 
    
    :var line: The configured changed line. 
    :var iceChanges: A list of changes in the line based on Incremental Change.
    :var configuredCompetingIces: A list of incremental change elements that are configured - yet not selectable. The
    best candidate appears in iceChanges. Currently not supported and all ice elements appear under iceChanges.
    """
    line: BOMLine = None
    iceChanges: List[BaseIceInfo] = ()
    configuredCompetingIces: List[BaseIceInfo] = ()


@dataclass
class LineOccInfo(TcBaseObj):
    """
    Details the changes in Occurrence Effectivity context for each BOMLine.
    
    :var line: The line affected by OccurrenceEffectivity.
    :var occChanges: Details of changes in the line based on OccurrenceEffectivity.
    """
    line: BOMLine = None
    occChanges: OccEffInfo = None


@dataclass
class LineRevInfo(TcBaseObj):
    """
    Details the changes in Revision Effectivity context for each BOMLine.
    
    :var line: The line affected by RevisionEffectivity.
    :var revChanges: Details of changes in the line based on RevisionEffectivity.
    """
    line: BOMLine = None
    revChanges: RevEffInfo = None


@dataclass
class OccEffInfo(TcBaseObj):
    """
    OccEffInfo structure details the changes in OccurrenceEffectivity context for each PSOccurrence configured by
    OccurrenceEffectivity.
    
    :var changeType: Type of occurrence change. Currently, there is only one type of change supported.
    - 1 means data has been configured based on OccurrenceEffectivity.
    
    
    :var occurrence: The object which is configured by OccurrenceEffectivity.
    :var effectivity: detail of occurrence effectivity.
    :var auxiliaryData: A generic structure of keys and vectors of values (integer,double,object,string,date types) for
    passing additional metadata regarding the nature of incremental change.
    """
    changeType: int = 0
    occurrence: BusinessObject = None
    effectivity: str = ''
    auxiliaryData: AdditionalInfo = None


@dataclass
class OccurrenceEffectivitySearchInfo(TcBaseObj):
    """
    OccurrenceEffectivitySearchInfo structure contains information for scoping the Occurrence Effectivity parameters
    used for tracking changes in structure based on OccurrenceEffectivity.
    
    :var isUnitData: If true unit data is being provided as opposed to date information.
    :var unitData: The details for searching changes based on unit information ( start Unit, end Unit and the end Item
    for the Unit effectivity).
    :var dateEffectivityData: The details for searching changes based on date (start Date and end Date).
    """
    isUnitData: bool = False
    unitData: EndItemUnitData = None
    dateEffectivityData: DateEffectivityData = None


@dataclass
class RevEffInfo(TcBaseObj):
    """
    RevEffInfo structure details the changes in RevisionEffectivity context for each ItemRevision configured by
    RevisionEffectivity.
    
    :var changeType: Type of revision change. Currently, there is only one type of change supported.
    - 1 means data has been configured based on RevisionEffectivity.
    
    
    :var revision: The object which is configured by RevisionEffectivity.
    :var howConfigured: detail of how the revision is configured.
    :var unitRange: unit range text if unit effectivity is used.
    :var dateRange: date range text if date effectivity is used.
    :var auxiliaryData: A generic structure of keys and vectors of values (integer,double,object,string,date types) for
    passing additional metadata regarding the nature of incremental change.
    """
    changeType: int = 0
    revision: BusinessObject = None
    howConfigured: str = ''
    unitRange: str = ''
    dateRange: str = ''
    auxiliaryData: AdditionalInfo = None


@dataclass
class RevisionEffectivitySearchInfo(TcBaseObj):
    """
    RevisionEffectivitySearchInfo structure contains information for scoping the Revision Effectivity  parameters used
    for tracking changes in structure based on RevisionEffectivity.
    
    :var owningUser: The owning user of the incremental change revision object(s) to search for.
    :var owningGroup: The owning group of the incremental change revision object(s) to search for.
    :var statuses: The details for status names and their start date.
    :var isUnitData: If true unit data is being provided as opposed to date information.
    :var unitData: The details for searching changes based on unit information ( start Unit, end Unit and the end Item
    for the Unit effectivity).
    :var dateEffectivityData: The details for searching changes based on date (start Date and end Date).
    """
    owningUser: str = ''
    owningGroup: str = ''
    statuses: StatusData = None
    isUnitData: bool = False
    unitData: EndItemUnitData = None
    dateEffectivityData: DateEffectivityData = None


@dataclass
class BaseIceInfo(TcBaseObj):
    """
    BaseIceInfo structure details the changes in Incremental Change context for each Incremental Change Element.
    
    :var typeOfIce: Type of incremental change.
    - 1 means data has been added in context of incremental change.
    - 2 means data has been removed in context of incremental change.
    
    
    :var howConfigured: detail of how the incremental change is configured. If 'howConfigured' is empty, it means the
    incremental change element is not configured.
    :var unitRange: unit range text. Empty if not unit effectivity.
    :var dateRange: date range text if date effectivity is used.
    :var absoccRootLine: The context line name in which the change occurred
    :var auxiliaryData: A generic structure of keys and vectors of values (integer,double,object,string,date types) for
    passing additional metadata regarding the nature of incremental change.
    :var icRev: incremetal change revision object which is referenced by the incremental change element associated with
    the change.
    :var ice: incremental change element associated with the change on the bomline. This change can be because of
    structural changes, attachment changes, predecessor changes or activity changes associated with the bomline.
    :var affectedObject: The object affected by the change. It can be one of the following types (not full list):
    - PSOccurrence
    - Datasets
    - Folders
    - Forms
    - Activities
    - AbsOccData
    
    
    :var attributeName: Name of the attribute associated with the change
    :var attributeDisplayName: Display name of the attribute whose value changed.
    :var attributeValue: Value of the attribute after change in Incremental change context.
    :var predecessorName: name of predecessor.
    :var predecessorSequence: Sequence number of predecessor.
    """
    typeOfIce: int = 0
    howConfigured: str = ''
    unitRange: str = ''
    dateRange: str = ''
    absoccRootLine: str = ''
    auxiliaryData: AdditionalInfo = None
    icRev: ItemRevision = None
    ice: IncrementalChangeElement = None
    affectedObject: BusinessObject = None
    attributeName: str = ''
    attributeDisplayName: str = ''
    attributeValue: str = ''
    predecessorName: str = ''
    predecessorSequence: str = ''


@dataclass
class StatusData(TcBaseObj):
    """
    StatusData structure contains filtering criterion for release status objects associated with the change.
    
    :var listOfStatus: A list of status names like Pending, TCM Released etc. to be used to filter based on status names
    :var sinceDate: The date on or after which the status objects are created.
    """
    listOfStatus: List[str] = ()
    sinceDate: datetime = None


@dataclass
class StructureChangesResponse(TcBaseObj):
    """
    StructureChangesResponse structure contains a vector of structureChangesResponseElement, the size of which matches
    the input ChangeTrackerInput vector
    
    :var responseElements: A list of StructureChangesResponseElement structures.
    :var serviceData: Service data capturing partial errors using the input array index as client id.
    """
    responseElements: List[StructureChangesResponseElement] = ()
    serviceData: ServiceData = None


@dataclass
class StructureChangesResponseElement(TcBaseObj):
    """
    StructureChangesResponseElement returns the changed lines and removed lines. The removed lines are configured in
    removed lines. Should treat the removed lines with caution as they won't show up in the configured bom. 
    
    :var addedLines: A list of changed lines.
    :var removedLines: A list of configured removed lines. This is separate even though it is a change, to allow the
    clients to treat the removed lines carefully ( no selection synchronization for example).
    :var visibleLinesByOccEff: A list of changed lines in context of Occurrence Effectivity.
    :var invisibleLinesByOccEff: A list of unconfigured changed lines in context of Occurrence Effectivity.
    :var visibleLinesByRevEff: A list of changed lines in context of Revision Effectivity.
    :var invisibleLinesByRevEff: A list of unconfigured changed lines in context of Revision Effectivity.
    """
    addedLines: List[LineIceInfo] = ()
    removedLines: List[LineIceInfo] = ()
    visibleLinesByOccEff: List[LineOccInfo] = ()
    invisibleLinesByOccEff: List[LineOccInfo] = ()
    visibleLinesByRevEff: List[LineRevInfo] = ()
    invisibleLinesByRevEff: List[LineRevInfo] = ()


@dataclass
class ChangeTrackerInput(TcBaseObj):
    """
    ChangeTrackerInput structure contains a vector of ChangeTrackerDataTypes representing the type of change to track
    (any unique combination of IncrementalChange, RevisionEffectivity, OccurrenceEffectivity), a vector of BOMLine
    objects that are used for scoping the data (only lines below the scope are considered for change tracking), and a
    structure representing the selection criteria for Incremental Change. Currently, only Incremental Change based
    changes are trackable.
    
    :var dataTypes: Types of changes to be tracked. A unique of set of ChangeTrackerDataTypes
    (IncrementalChange,RevisionEffectivity,OccurrenceEffectivity). Currently, only IncrementalChange is supported.
    :var selectedLines: A list of scope lines below which the changes are to be tracked. Currently, must be BOMLine
    objects or their subtypes.
    :var occEffInfo: Occurrence effectivity selection criterion. Captures information about range of units/dates to be
    considered while searching for Occurrence effectivities.
    :var revEffInfo: Revision effectivity selection criterion. Captures information about range of units/dates/statuses
    to be considered while searching for Revision effectivities.
    :var icInfo: Incremental change context selection criterion. Captures information about range of
    units/dates/intents/statuses to be considered while searching for IncrementalChanges.
    """
    dataTypes: List[ChangeTrackerDataType] = ()
    selectedLines: List[BusinessObject] = ()
    occEffInfo: OccurrenceEffectivitySearchInfo = None
    revEffInfo: RevisionEffectivitySearchInfo = None
    icInfo: ICSearchInfo = None


@dataclass
class DateEffectivityData(TcBaseObj):
    """
    DateEffectivtyData structure captures information about date based effectivity. The search will track changes
    between the start Date and end Date. End Date must be greater than start Date.
    
    :var startDate: The start date of the range to search in.
    :var endDate: The end date of the range to search in.
    """
    startDate: datetime = None
    endDate: datetime = None


@dataclass
class AdditionalInfo(TcBaseObj):
    """
    a generic structure to capture additional information.
    
    :var intMap: A map (string/list of integers) of generic key to integer values.
    :var dblMap: A map (string/list of doubles) of generic key to double values.
    :var strMap: A map (string/list of strings) of generic key to string values.
    :var objMap: A map (string/list of BusinessObjects) of generic key to  BusinessObject values.
    :var dateMap: A map (string/list of dates) of generic key to date values.
    """
    intMap: StringToIntVectorMap = None
    dblMap: StringToDblVectorMap = None
    strMap: StringtoStrVectorMap = None
    objMap: StringToObjVectorMap = None
    dateMap: StringToDateVectorMap = None


@dataclass
class EndItemUnitData(TcBaseObj):
    """
    EndItemUnitData structure captures information about unit based effectivity. The search will track changes between
    the start Unit and end Unit for the specified End Item. All 3 must be valid for successful usage.
    
    :var startUnit: The start unit. Valid values must be 0 or greater.
    :var endUnit: The end Unit. Valid values must be greater than startUnit.
    :var endItem: Item or ItemRevision representing the end item corresponding to the start and end unit range.
    """
    startUnit: int = 0
    endUnit: int = 0
    endItem: BusinessObject = None


class ChangeTrackerDataType(Enum):
    """
     enum  ChangeTrackerDataType {IncrementalChange, RevisionEffectivity, OccurrenceEffectivity, Time};
    
    - Incremental Change     Track incremental Change based changes.
    - RevisionEffectivity         Track RevisionEffectivity based changes ( currently not supported ).
    - OccurrenceEffectivity     Track OccurrencEffectivity based changes ( currently not supported).
    - Time                         Track changes given a date range based on modified times of objects in that range (
    Currently not supported ).
    
    """
    IncrementalChange = 'IncrementalChange'
    RevisionEffectivity = 'RevisionEffectivity'
    OccurrenceEffectivity = 'OccurrenceEffectivity'
    Time = 'Time'


"""
a map of string to vector of dates
"""
StringToDateVectorMap = Dict[str, List[datetime]]


"""
String to vector of doubles map.
"""
StringToDblVectorMap = Dict[str, List[float]]


"""
map of string to vector of integers.
"""
StringToIntVectorMap = Dict[str, List[int]]


"""
a map of string to vector of objects.
"""
StringToObjVectorMap = Dict[str, List[BusinessObject]]


"""
A map of string to vector of strings.
"""
StringtoStrVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2008_06.Structure import ValidateStructureItemIdsResponse, ValidateStructureItemIdsInfo, ExpandOrUpdateDuplicateItemsResponse, ExpandOrUpdateDuplicateItemsInfo, DuplicateResponse, DuplicateInputInfo
from typing import List
from tcsoa.base import TcService


class StructureService(TcService):

    @classmethod
    def validateStructureItemIds(cls, inputs: List[ValidateStructureItemIdsInfo]) -> ValidateStructureItemIdsResponse:
        """
        This operation will validate the un-validated ItemIds that will be used to clone(duplicate) an assembly
        structure.  All portions of the structure that are displayed in the duplicate dialog have been validated by the
        client.  
        
        Note: The differences between the three operations 'validateStructureItemIds', 'validateStructureItemIds2' and
        'validateStructureItemIds3' are the following:
        - In 'validateStructureItemIds' the input is the top BOMLine of the original configured structure in Structure
        Manager.  In 'validateStructureItemIds2' the input is the selected BOMLine of the configured structure in
        Structure Manager or Systems Engineering.  i.e. the user can select to clone a sub-assembly of the original
        structure or the entire assembly.  The input for 'validateStructureItemIds2' includes project(s).  The cloned
        structure is assigned to those project(s).  'validateStructureItemIds' did not have projects as input.
        
        - 'validateStructureItemIds3' was created as a result of the Multi Field Key(MFK) project.  The difference
        between 'validateStructureItemIds2' and 'validateStructureItemIds3' is that in 'validateStructureItemIds2' the
        input and output have a 'DuplicateIdMap2' whereas to align with the MFK project, 'validateStructureItemIds3'
        has a 'DuplicateIdStructure'.
        
        
        Use cases:
        The user sends in a structure for validation of the new ItemIds .  The input to this operation is the top
        BOMLine, a map of old ItemIds to new ones for those Item objects that are already validated, and the default
        naming scheme.  Based on the structure traversal, the input map, and the naming scheme this operation validates
        whether the proposed names for the un-validated Item objects are valid.
        """
        return cls.execute_soa_method(
            method_name='validateStructureItemIds',
            library='StructureManagement',
            service_date='2008_06',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=ValidateStructureItemIdsResponse,
        )

    @classmethod
    def duplicate(cls, inputs: List[DuplicateInputInfo]) -> DuplicateResponse:
        """
        This operation will create a duplicate (clone) of the input structure from its top level down.
        Depending on the user action, all or some of the original structure is duplicated and the rest is referenced. 
        The user has the option to de-select a sub assembly or leaf Item on the duplicate dialog.  Those Item objects
        that are not selected in the duplicate dialog will not be cloned but referenced from the original structure.
        
        The user can define a specific naming pattern for the ItemIds of the duplicated (cloned) structure.  The user
        can define specific ItemIds for individually selected ItemRevision objects or a default naming pattern for the
        duplicated structure.   The default pattern can be defined by adding prefixes, suffixes or replacing part of
        the original name with a different pattern.  The user can also choose to allow the system to assign default ids.
        
        All of the structure dependent data of the input structure like datasets and attachments are copied to the
        duplicated structure based on the Business Modeler IDE deep copy rules for 'SaveAs'.  The duplicate operation
        internally uses 'SaveAs' at every level of the structure; therefore it uses the 'SaveAs' deep copy rules.
        
        CAD specific attachments and relations are copied based on the transfer mode defined for the Business Modeler
        IDE global constant StructureCloneTransferModes.  The transfer mode contains dependent closure rules for
        expansion and cloning.  The value for the closure rules is added by the installed CAD system.
        
        For e.g. The closure rule defined for IPEM ProE integration (ipemSCloneClosureRule) looks like this:
        
        TYPE.ProPrt:CLASS.ItemRevision:RELATIONP2S.IPEM_master_dependency:PROCESS:PartFamilyMaster:R,
        TYPE.ProAsm:CLASS.ItemRevision:RELATIONP2S.IPEM_master_dependency:PROCESS:PartFamilyMaster:R,
        TYPE.ProPrt:CLASS.ItemRevision:RELATIONS2P.IPEM_master_dependency:PROCESS+TRAVERSE:PartFamilyMember, 
        
        Note: The difference between the three operations 'duplicate', 'duplicate2' and 'duplicate3' are the following:
        
        'duplicate' and 'duplicate2'
         - In 'duplicate' the input top BOMLine is the top BOMLine of the original configured structure in Structure
        Manager.  In 'duplicate2' the input top BOMLine is the selected BOMLine from the configured structure in
        Structure Manager or Systems Engineering.  i.e. the user can select to clone a sub-assembly of the original
        structure.
        
        - The input for 'duplicate2' includes project(s).  The cloned structure is assigned to those project(s). 
        'duplicate' does not have project(s) as input.
        
        'duplicate2' and 'duplicate3'
        - 'duplicate3' was created as a result of the Multi Field Key (MFK ) project.   The difference between
        'duplicate2' and 'duplicate3' is that in 'duplicate2' is that the input and output had a 'DuplicateIdMap2'
        whereas to align with the MFK project, 'duplicate3' has a 'DuplicateIdStructure'.
        
        
        Use cases:
        Use case1: Simple Clone
         A user has an assembly which does not have cad dependencies nor does it belong to a specific project(s).  The
        user wants to duplicate the entire structure with a specific naming pattern for the ItemIds.  The naming
        pattern is a prefix "test_".
        
        The user invokes the duplicate operation by passing in the top BOMLine of the structure to be cloned, and the
        naming pattern for the new structure.  The result is a new structure with the following naming pattern for the
        ItemIds -> test_OriginalItemId.
        
        Use case2: CAD Clone
        A user has an assembly structure which has cad dependencies.  The user wants to start a new product with a
        similar structure and cad dependencies.  The expansion and cloning rules have been defined in the Business
        Modeler IDE global constant StructureCloneTransferModes 
        The user invokes the duplicate operation by passing in the top BOMLine of the structure to be cloned.  
        The user picks the cad dependency option "'Part Families Masters'".  The expansion and cloning will be done
        based on the closure rules defined for Part Family Master in the CAD closure rules.
        The '"Rename Cad Files'" will be passed to the CAD integration in a callback.  If the '"Rename Cad Files'", is
        set to true by the user, the cad files need to be renamed by the cad integration.
        The result will be a duplicated structure with the expected cad dependencies and it will open in the CAD tool
        without any errors.
        """
        return cls.execute_soa_method(
            method_name='duplicate',
            library='StructureManagement',
            service_date='2008_06',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=DuplicateResponse,
        )

    @classmethod
    def expandOrUpdateDuplicateItems(cls, infos: List[ExpandOrUpdateDuplicateItemsInfo]) -> ExpandOrUpdateDuplicateItemsResponse:
        """
        This operation is called as part of the duplicate functionality.   It expands the structure one level at a time
        and gets structure dependent data. The dependencies have been defined to allow duplication of CAD dependent
        structure.  The 'expandOrUpdateDuplicateItems' operation uses Business Modeler IDE global constant
        StructureCloneTransferModes to determine which of the cad specific attachments and relationships can be
        expanded. This constant defines the transfer modes containing dependent closure rules for expansion and
        cloning.  The value for the closure rules is added by the installed CAD system.
        
        Note: The difference between 'expandOrUpdateDuplicateItems', 'expandOrUpdateDuplicateItems2' and
        'expandOrUpdateDuplicateItems3' are as follows:
        
        Difference in 'expandOrUpdateDuplicateItems' and 'expandOrUpdateDuplicateItems2'
        -    We allow the user to select a sub assembly for duplication.  There is a performance problem that was
        uncovered.  Even though a sub assembly is sent for duplication, traversal on the server side was happening from
        the top BOMLine of the structure.  To improve the performance we get a BOB object.
        
        -    The smart selection logic was added to decide whether to clone or reference an Item in a structure based
        on the project that the top line of the original structure belongs to.  This smart selection logic is bottom
        up, so if a child is selected based on project assignment, the parent will be selected, no matter whether the
        parent belongs to the top item project assignment or not.
        
        Difference in 'expandOrUpdateDuplicateItems2' and 'expandOrUpdateDuplicateItems3'
        The mandatory flag is passed back to the client.  When a cad option has been flagged with a mandatory flag, a
        "R" predicate in the closure rules, that option will come up in the Duplicate Dialog pre-checked and its
        selected status will be un-changeable.  This will prevent the user from un-checking those ItemRevision objects
        that were added in to make the structure cad consistent.  That is if a family table member in the assembly
        structure has been selected for duplication, then automatically include all its masters from the referenced
        ItemRevision all the way to the top master.  Including the masters automatically will only happen if the CAD
        closure rules are defined with a predicate "R" that says this CAD relation is mandatory for duplication.
        
        
        Use cases:
        The user sends in a structure for expansion, it will be expanded one level at a time and all dependent data
        will be returned based on the input and the value of the closure rules defined.  The input consists of the
        BOMLine for expansion, the ItemRevision objects on which to perform the expansion, and the dependency types. 
        The ItemRevision objects could be null, in which case the ItemRevision object(s) gotten from the expansion of
        the BOMLine are used.  The dependency types are checked against the definition in the closure rules to
        determine what dependent data is expanded.
        """
        return cls.execute_soa_method(
            method_name='expandOrUpdateDuplicateItems',
            library='StructureManagement',
            service_date='2008_06',
            service_name='Structure',
            params={'infos': infos},
            response_cls=ExpandOrUpdateDuplicateItemsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, POM_object, PSBOMViewRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class NetEffectivityOutput(TcBaseObj):
    """
    Holds unit net effectivity data for each BOMLine object.
    
    :var occEffectivity: The occurrence effectivity of the BOMLine
    :var netEffectivity: The calculated unit net effectivity of the BOMLine
    :var netEOC: Indicates whether the BOMLine would configure based on its calculated unit net effectivity
    """
    occEffectivity: str = ''
    netEffectivity: str = ''
    netEOC: bool = False


@dataclass
class NetEffectivityResponse(TcBaseObj):
    """
    Holds the NetEffectivity output response for the input BOMLine objects.
    
    :var netEffectivityOutputVector: Holds unit net effectivity data for input BOMLine objects.
    :var serviceData: The Service Data.
    """
    netEffectivityOutputVector: List[NetEffectivityOutput] = ()
    serviceData: ServiceData = None


@dataclass
class OccCutbackValueInfo(TcBaseObj):
    """
    Contains  effectivity value proposals for a specified occurrence
    
    :var occ: occurrence whose effectivity is being modified, may not be null
    :var effProposals: list of proposed effectivity values, may be empty
    """
    occ: BusinessObject = None
    effProposals: List[ProposalInfo] = ()


@dataclass
class OccEffCutbackInfo(TcBaseObj):
    """
    Contains unit occurrence effectivity cutback information for the specified PSBOMViewRevision
    
    :var cutbackParent: PSBOMViewRevision object for which to manage cutback data
    :var cutbacks: list of cutback objects, may be empty
    """
    cutbackParent: PSBOMViewRevision = None
    cutbacks: List[CutbackInfo] = ()


@dataclass
class ProposalInfo(TcBaseObj):
    """
    Contains the Teamcenter:: Fnd0OccEffValue proposal being modified and the proposed effectivity value, in format
    unit range(enditem). The proposal may not be null, an empty effectivity value  allows re-generation of a proposal
    value.
    
    :var proposal: Teamcenter:: Fnd0OccEffValue proposal being modified, may not be null
    :var effectivity: proposed effectivity value, in format: unit range(enditem), if empty then allows re-generation of
    proposal value
    """
    proposal: POM_object = None
    effectivity: str = ''


@dataclass
class CutbackInfo(TcBaseObj):
    """
    Contains data to govern the cuback action. The solution BOMLine objects take on the cutback effectivity, and the
    impacted BOMLine objects may not be effective for that cutback effectivity.
    
    :var cutback: Teamcenter::Fnd0OccEffCutback cutback being modified, if null then create a new cutback object
    :var name: name of cutback, may not be null
    :var description: description of cutback, may be null
    :var sequence: sequence order during application of active cutbacks
    :var effectivity: list of effectivity values to drive cutback action, each element in format: unit range(enditem)
    :var solutions: list of OccCutbackValueInfo objects for the cutback effectivity, may be empty
    :var impacted: list of OccCutbackValueInfo objects to be impacted by solution effectivity, may not be empty
    :var allowGaps: indicates whether proposals can leave gaps in overall effectivity
    :var actionForCutback: The action to take on the cutback object: 1=create, 2=modify, 3=delete
    """
    cutback: POM_object = None
    name: str = ''
    description: str = ''
    sequence: int = 0
    effectivity: List[str] = ()
    solutions: List[OccCutbackValueInfo] = ()
    impacted: List[OccCutbackValueInfo] = ()
    allowGaps: bool = False
    actionForCutback: int = 0


@dataclass
class CutbackUnitEffectivityResponse(TcBaseObj):
    """
    Unit occurrence effectivity cutback data for input BOMLine objects.
    
    :var cutbacks: Holds unit occurrence effectivity cutback data for input PSBOMViewRevision objects
    :var serviceData: The Service Data
    """
    cutbacks: List[OccEffCutbackInfo] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AMImpactedObjectsResponse(TcBaseObj):
    """
    Data structure that holds the UIDs of Teamcenter business objects whose READ access is impacted by the Access
    Manager rule tree changes, a Boolean flag that indicates if full reindex is needed or not and the service data with
    any partial errors that might happen while finding the impacted objects
    
    :var serviceData: serviceData
    :var isImpactGlobal: isImpactGlobal
    :var impactedUids: impactedUids
    """
    serviceData: ServiceData = None
    isImpactGlobal: bool = False
    impactedUids: List[str] = ()


@dataclass
class GetSessionValuesResponse(TcBaseObj):
    """
    Structure that holds the SessionValuesMap which contains session keys and corresponding session values. Each
    session key in the SessionValuesMap corresponds to a session attribute like groups, roles and project teams.
    
    :var sessionValues: A map of key-value pairs that holds the session keys and the matching session values from Read
    Expression string.
    """
    sessionValues: SessionValuesMap = None


"""
List of session names which maps session name to value(s). All values are expressed in their string equivalent.
"""
SessionValuesMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Internal.Multisite._2017_11.ArchiveRestore import ArchiveRestoreResponse, GetReportTicketResponse, NamesAndValues
from tcsoa.gen.BusinessObjects import BusinessObject, PublishedObject
from typing import List
from tcsoa.base import TcService


class ArchiveRestoreService(TcService):

    @classmethod
    def getReportFileTicket(cls, reportFileName: str) -> GetReportTicketResponse:
        """
        The operation generates a File Management System (FMS) transient read ticket for report file created by
        archive/restore background operation.
        
        Use cases:
        In the Multi-Site federation user can archive and restore  the business objects  in background mode using Rich
        Application Client. This frees up the RAC to do other tasks. A non-modal progress monitor gives frequent
        updates of the operation progress and  archive/restore operation completion report.
        """
        return cls.execute_soa_method(
            method_name='getReportFileTicket',
            library='Internal-Multisite',
            service_date='2017_11',
            service_name='ArchiveRestore',
            params={'reportFileName': reportFileName},
            response_cls=GetReportTicketResponse,
        )

    @classmethod
    def restoreObjects(cls, objects: List[PublishedObject], sessionOptionsAndValues: List[NamesAndValues]) -> ArchiveRestoreResponse:
        """
        This operation transfers the ownership of published objects representing the target objects  at archive site to
        the local site using the Multi-Site TCXML payload. Further it un-publishes the target objects at archive site
        and deletes the target object replicas at archive site. The options specified by NamesAndValues  structure
        affect the final state of the restored objects at the local sites.
        
        Use cases:
        In the Multi-Site federation user can restore the business objects by performing ownership transfer from
        archive site to local site.  After the successful ownership data transfer the replicated data in the archive
        site will be deleted.
        """
        return cls.execute_soa_method(
            method_name='restoreObjects',
            library='Internal-Multisite',
            service_date='2017_11',
            service_name='ArchiveRestore',
            params={'objects': objects, 'sessionOptionsAndValues': sessionOptionsAndValues},
            response_cls=ArchiveRestoreResponse,
        )

    @classmethod
    def archiveObjects(cls, objects: List[BusinessObject], excludeObjects: List[BusinessObject], sessionOptionsAndValues: List[NamesAndValues]) -> ArchiveRestoreResponse:
        """
        This operation transfers the ownership of  targeted objects from the local site to the archive site using the
        Multi-Site TCXML payload and publishes them at archive site. The target objects belonging to local site will 
        be archived. After the successful ownership data transfer the replicated data and published records in the
        participating sites in a Multi-Site federation will be deleted. The options specified by NamesAndValues
        structure affect the final state of the  archived objects at the target sites.
        
        Use cases:
        In the Multi-Site federation user can archive the business objects by performing ownership transfer to a remote
        site marked for archive purpose.  After the successful ownership data transfer the replicated data in the local
        site will be deleted.
        """
        return cls.execute_soa_method(
            method_name='archiveObjects',
            library='Internal-Multisite',
            service_date='2017_11',
            service_name='ArchiveRestore',
            params={'objects': objects, 'excludeObjects': excludeObjects, 'sessionOptionsAndValues': sessionOptionsAndValues},
            response_cls=ArchiveRestoreResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetVPSRConditionsResponse(TcBaseObj):
    """
    Response structure for service operation 'getVPSRConditions'.
    
    :var condData: It is a list of 'ConditionData' structres which contain information of the conditions found.
    :var serviceData: Standard 'ServiceData' structure. It contains partial errors and failures along with the
    clientIds.
    """
    condData: List[ConditionData] = ()
    serviceData: ServiceData = None


@dataclass
class ConditionData(TcBaseObj):
    """
    A structure containing name and expression of a condition entry.
    
    :var condName: The name of condition.
    :var condExpress: The expression of condition.
    """
    condName: str = ''
    condExpress: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import EPMJob, EPMTaskTemplate
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ApplyTemplateInput(TcBaseObj):
    """
    New versions of the workflow templates that needs to be applied including the corresponding client ids.
    
    :var clientId: client id
    :var processTemplate: Process template to be applied
    """
    clientId: str = ''
    processTemplate: EPMTaskTemplate = None


@dataclass
class ApplyTemplateOutput(TcBaseObj):
    """
    Results from applying a template to its corresponding active processes.
    
    :var clientId: client id
    :var updatedProcesses: Active processes that were updated successfully
    :var failedProcesses: Active processes that could not be updated
    """
    clientId: str = ''
    updatedProcesses: List[EPMJob] = ()
    failedProcesses: List[EPMJob] = ()


@dataclass
class ApplyTemplateResponse(TcBaseObj):
    """
    Information about active processes that were updated with template changes
    
    :var applyTemplateOutput: List of processes that were updated and list of processes that failed.
    :var serviceData: Service Data
    """
    applyTemplateOutput: List[ApplyTemplateOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanFile, Dataset
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FileTicketsResponse(TcBaseObj):
    """
    Holds a 'ServiceData' object and a map (ImanFile, string) of read tickets.  If the request completes successfully,
    each input ImanFile object will appear as a key in the output map
    and the string value associated with each entry will be a valid FMS ticket associated with that ImanFile.  Returned
    from the 'getFileReadTickets' operation.
    
    :var tickets: A map of the input ImanFile objects to FMS tickets used to access files in the Teamcenter volume.
    :var serviceData: The Teamcenter Services structure used to return status of the operation.  Any errors that
    occurred during the operation are returned here.
    """
    tickets: TicketMap = None
    serviceData: ServiceData = None


@dataclass
class GetDatasetWriteTicketsInputData(TcBaseObj):
    """
    A vector of 'GetDatasetWriteTicketsInputData' structures which holds the Dataset objects and related
    'DatasetFileTicketInfo'.  The calling client must construct this input.
    
    :var dataset: The Dataset object to which the ImanFile object(s) representing the data files uploaded to the
    Teamcenter volume are associated.
    :var createNewVersion: If this flag is true, a new DatasetRevision will be created to reference the new set of
    files uploaded to the Teamcenter volume.  Any existing DatasetRevision will be unaltered, but the new
    DatasetRevision will have the files attached as named references.
    If this flag is false, the uploaded files are attached to the current most recent DatasetRevision of this Dataset
    as named references.
    
    :var datasetFileInfos: A vector of 'DatasetFileInfo' structures where each contains information specific to each
    Dataset to be created.
    """
    dataset: Dataset = None
    createNewVersion: bool = False
    datasetFileInfos: List[DatasetFileInfo] = ()


@dataclass
class GetDatasetWriteTicketsResponse(TcBaseObj):
    """
    Holds a 'ServiceData' object and a vector of dataset commit information.  If the request completes successfully,
    the information in the vector can be used to commit the Dataset to the database.
    
    :var commitInfo: A vector of 'CommitDatasetFileInfo' structures.
    :var serviceData: The Teamcenter Services structure used to return status of the operation.  Any errors that
    occurred during the operation are returned here.
    """
    commitInfo: List[CommitDatasetFileInfo] = ()
    serviceData: ServiceData = None


@dataclass
class CommitDatasetFileInfo(TcBaseObj):
    """
    Used in the 'commitDatasetFiles' operation to pass information for committing ImanFile instances uploaded to a
    Teamcenter volume with the associated Dataset instances.
    
    :var dataset: The Dataset object to which the ImanFile object(s) representing the data files uploaded to the
    Teamcenter volume.
    :var createNewVersion: If this flag is true, a new DatasetRevision will be created to reference the new set of
    files uploaded to the Teamcenter volume.  Any existing DatasetRevision will be unaltered, but the new
    DatasetRevision will have the files attached as named references.
    If this flag is false, the uploaded files are attached to the current most recent DatasetRevision of this Dataset
    as named references.
    
    :var datasetFileTicketInfos: A vector of DatasetFileTicketInfo objects representing each ImanFile object attached
    to the given Dataset as a named reference.
    """
    dataset: Dataset = None
    createNewVersion: bool = False
    datasetFileTicketInfos: List[DatasetFileTicketInfo] = ()


@dataclass
class DatasetFileInfo(TcBaseObj):
    """
    The structure defining the basic information for a file to be uploaded to a Dataset.
    
    :var clientId: An identifier defined by the caller to track the related object.
    
    :var fileName: The name of the file to be uploaded. 
    This is the filename only and should not contain the path to the filename.
    Exception: When used with the 'FileManagementUtility::putFiles()' operation, this should contain the full path and
    file name.
    :var namedReferencedName: The named reference relation to the file.
    :var isText: A flag to indicate if the file is text (TRUE) or binary (FALSE).
    :var allowReplace: A flag to indicate if the file may be overwritten (TRUE) or not (FALSE).
    """
    clientId: str = ''
    fileName: str = ''
    namedReferencedName: str = ''
    isText: bool = False
    allowReplace: bool = False


@dataclass
class DatasetFileTicketInfo(TcBaseObj):
    """
    A structure representing an ImanFile object attached to a given Dataset as a named reference.
    
    :var datasetFileInfo: The structure defining the basic information for a file to be uploaded to a Dataset.
    :var ticket: The FMS write ticket used to transfer the file to the appropriate Teamcenter volume.
    """
    datasetFileInfo: DatasetFileInfo = None
    ticket: str = ''


"""
A map of the input ImanFile objects to FMS read tickets used to read the file from the Teamcenter volume.
"""
TicketMap = Dict[ImanFile, str]



from tcsoa.gen.Core._2014_10.services import DataManagementService as imp0
from tcsoa.gen.Core._2020_01.services import ProjectLevelSecurityService as imp1
from tcsoa.gen.Core._2008_06.services import DataManagementService as imp2
from tcsoa.gen.Core._2014_06.services import DigitalSignatureService as imp3
from tcsoa.gen.Core._2007_09.services import ProjectLevelSecurityService as imp4
from tcsoa.gen.Core._2017_05.services import ProjectLevelSecurityService as imp5
from tcsoa.gen.Core._2014_06.services import ReservationService as imp6
from tcsoa.gen.Core._2012_02.services import DataManagementService as imp7
from tcsoa.gen.Core._2006_03.services import ReservationService as imp8
from tcsoa.gen.Core._2006_03.services import DataManagementService as imp9
from tcsoa.gen.Core._2018_11.services import ProjectLevelSecurityService as imp10
from tcsoa.gen.Core._2006_03.services import FileManagementService as imp11
from tcsoa.gen.Core._2017_05.services import FileManagementService as imp12
from tcsoa.gen.Core._2008_03.services import SessionService as imp13
from tcsoa.gen.Core._2012_09.services import ProjectLevelSecurityService as imp14
from tcsoa.gen.Core._2007_12.services import DataManagementService as imp15
from tcsoa.gen.Core._2016_09.services import DataManagementService as imp16
from tcsoa.gen.Core._2010_04.services import DataManagementService as imp17
from tcsoa.gen.Core._2008_06.services import DispatcherManagementService as imp18
from tcsoa.gen.Core._2020_01.services import DataManagementService as imp19
from tcsoa.gen.Core._2008_06.services import StructureManagementService as imp20
from tcsoa.gen.Core._2018_06.services import DataManagementService as imp21
from tcsoa.gen.Core._2007_01.services import DataManagementService as imp22
from tcsoa.gen.Core._2010_09.services import DataManagementService as imp23
from tcsoa.gen.Core._2015_07.services import DataManagementService as imp24
from tcsoa.gen.Core._2007_01.services import ManagedRelationsService as imp25
from tcsoa.gen.Core._2007_06.services import DataManagementService as imp26
from tcsoa.gen.Core._2007_09.services import DataManagementService as imp27
from tcsoa.gen.Core._2016_05.services import DataManagementService as imp28
from tcsoa.gen.Core._2020_04.services import DataManagementService as imp29
from tcsoa.gen.Core._2013_05.services import DataManagementService as imp30
from tcsoa.gen.Core._2010_04.services import LanguageInformationService as imp31
from tcsoa.gen.Core._2007_06.services import LOVService as imp32
from tcsoa.gen.Core._2007_06.services import PropDescriptorService as imp33
from tcsoa.gen.Core._2011_06.services import PropDescriptorService as imp34
from tcsoa.gen.Core._2006_03.services import SessionService as imp35
from tcsoa.gen.Core._2011_06.services import SessionService as imp36
from tcsoa.gen.Core._2008_06.services import PropDescriptorService as imp37
from tcsoa.gen.Core._2012_10.services import DataManagementService as imp38
from tcsoa.gen.Core._2012_02.services import OperationDescriptorService as imp39
from tcsoa.gen.Core._2016_10.services import ProjectLevelSecurityService as imp40
from tcsoa.gen.Core._2015_10.services import FileManagementService as imp41
from tcsoa.gen.Core._2008_06.services import SessionService as imp42
from tcsoa.gen.Core._2013_05.services import LOVService as imp43
from tcsoa.gen.Core._2009_10.services import DataManagementService as imp44
from tcsoa.gen.Core._2011_06.services import LOVService as imp45
from tcsoa.gen.Core._2017_11.services import LogicalObjectService as imp46
from tcsoa.gen.Core._2018_06.services import LogicalObjectService as imp47
from tcsoa.gen.Core._2018_11.services import LogicalObjectService as imp48
from tcsoa.gen.Core._2008_06.services import ManagedRelationsService as imp49
from tcsoa.gen.Core._2007_01.services import SessionService as imp50
from tcsoa.gen.Core._2010_04.services import SessionService as imp51
from tcsoa.gen.Core._2011_06.services import OperationDescriptorService as imp52
from tcsoa.gen.Core._2011_06.services import DataManagementService as imp53
from tcsoa.gen.Core._2014_06.services import DataManagementService as imp54
from tcsoa.gen.Core._2007_01.services import FileManagementService as imp55
from tcsoa.gen.Core._2015_10.services import SessionService as imp56
from tcsoa.gen.Core._2009_10.services import ProjectLevelSecurityService as imp57
from tcsoa.gen.Core._2019_06.services import SessionService as imp58
from tcsoa.gen.Core._2009_04.services import ProjectLevelSecurityService as imp59
from tcsoa.gen.Core._2011_06.services import ReservationService as imp60
from tcsoa.gen.Core._2014_10.services import ProjectLevelSecurityService as imp61
from tcsoa.gen.Core._2015_10.services import DataManagementService as imp62
from tcsoa.gen.Core._2007_06.services import SessionService as imp63
from tcsoa.gen.Core._2012_02.services import SessionService as imp64
from tcsoa.gen.Core._2012_09.services import DataManagementService as imp65
from tcsoa.gen.Core._2011_06.services import EnvelopeService as imp66
from tcsoa.gen.Core._2007_12.services import SessionService as imp67
from tcsoa.gen.Core._2009_04.services import SessionService as imp68
from tcsoa.gen.Core._2008_06.services import ReservationService as imp69
from tcsoa.gen.Core._2019_06.services import DataManagementService as imp70
from tcsoa.gen.Core._2008_05.services import DataManagementService as imp71
from tcsoa.base import TcService


class DataManagementService(TcService):
    addChildren = imp0.addChildren
    addParticipants = imp2.addParticipants
    bulkCreateObjects = imp7.bulkCreateObjects
    changeOwnership = imp9.changeOwnership
    createAlternateIdentifiers = imp15.createAlternateIdentifiers
    createAttachAndSubmitObjects = imp16.createAttachAndSubmitObjects
    createConnections = imp2.createConnections
    createDatasets = imp9.createDatasets
    createDatasets2 = imp17.createDatasets
    createDatasets3 = imp2.createDatasets2
    createFolders = imp9.createFolders
    createIdDisplayRules = imp19.createIdDisplayRules
    createItems = imp9.createItems
    createObjects = imp2.createObjects
    createObjectsInBulkAndRelate = imp21.createObjectsInBulkAndRelate
    createOrUpdateForms = imp22.createOrUpdateForms
    createOrUpdateGDELinks = imp2.createOrUpdateGDELinks
    createOrUpdateItemElements = imp2.createOrUpdateItemElements
    createOrUpdateRelations = imp2.createOrUpdateRelations
    createOrUpdateStaticTableData = imp23.createOrUpdateStaticTableData
    createRelateAndSubmitObjects2 = imp24.createRelateAndSubmitObjects2
    createRelations = imp9.createRelations
    deleteObjects = imp9.deleteObjects
    deleteRelations = imp9.deleteRelations
    expandGRMRelationsForPrimary = imp26.expandGRMRelationsForPrimary
    expandGRMRelationsForPrimary2 = imp27.expandGRMRelationsForPrimary
    expandGRMRelationsForSecondary = imp26.expandGRMRelationsForSecondary
    expandGRMRelationsForSecondary2 = imp27.expandGRMRelationsForSecondary
    findDisplayableSubBusinessObjects = imp2.findDisplayableSubBusinessObjects
    findDisplayableSubBusinessObjectsWithDisplayNames = imp17.findDisplayableSubBusinessObjectsWithDisplayNames
    generateContextSpecificIDs = imp28.generateContextSpecificIDs
    generateContextSpecificIDs2 = imp29.generateContextSpecificIDs2
    generateIdsUsingIDGenerationRules = imp0.generateIdsUsingIDGenerationRules
    generateItemIdsAndInitialRevisionIds = imp9.generateItemIdsAndInitialRevisionIds
    generateNextValues = imp30.generateNextValues
    generateNextValuesForProperties = imp24.generateNextValuesForProperties
    generateRevisionIds = imp9.generateRevisionIds
    generateUID = imp22.generateUID
    getAvailableTypes = imp26.getAvailableTypes
    getAvailableTypesWithDisplayNames = imp17.getAvailableTypesWithDisplayNames
    getChildren = imp30.getChildren
    getContextsAndIdentifierTypes = imp15.getContextsAndIdentifierTypes
    getCreatbleSubBuisnessObjectNames = imp24.getCreatbleSubBuisnessObjectNames
    getDatasetCreationRelatedInfo = imp22.getDatasetCreationRelatedInfo
    getDatasetCreationRelatedInfo2 = imp17.getDatasetCreationRelatedInfo2
    getDatasetTypeInfo = imp26.getDatasetTypeInfo
    getDatasetTypesWithFileExtension = imp38.getDatasetTypesWithFileExtension
    getDeepCopyData = imp0.getDeepCopyData
    getDeepCopyData2 = imp24.getDeepCopyData
    getDomainOfObjectOrType = imp24.getDomainOfObjectOrType
    getEventTypes = imp23.getEventTypes
    getIdContexts = imp19.getIdContexts
    getIdentifierTypes = imp19.getIdentifierTypes
    getItemAndRelatedObjects = imp2.getItemAndRelatedObjects
    getItemCreationRelatedInfo = imp22.getItemCreationRelatedInfo
    getItemFromAttribute = imp44.getItemFromAttribute
    getItemFromId = imp22.getItemFromId
    getLocalizedProperties = imp17.getLocalizedProperties
    getLocalizedProperties2 = imp24.getLocalizedProperties2
    getNRPatternsWithCounters = imp2.getNRPatternsWithCounters
    getNextIds = imp2.getNextIds
    getPasteRelations = imp30.getPasteRelations
    getPasteRelations2 = imp0.getPasteRelations2
    getProperties = imp9.getProperties
    getRevNRAttachDetails = imp2.getRevNRAttachDetails
    getStaticTableData = imp23.getStaticTableData
    getSubTypeNames = imp30.getSubTypeNames
    getTableProperties = imp44.getTableProperties
    getTraceReport = imp53.getTraceReport
    getTraceReport2 = imp38.getTraceReport
    getTraceReport3 = imp54.getTraceReport2
    getTraceReportLegacy = imp54.getTraceReportLegacy
    isPropertyLocalizable = imp17.isPropertyLocalizable
    listAlternateIdDisplayRules = imp15.listAlternateIdDisplayRules
    loadObjects = imp27.loadObjects
    moveToNewFolder = imp22.moveToNewFolder
    postEvent = imp23.postEvent
    pruneNamedReferences = imp0.pruneNamedReferences
    purgeSequences = imp26.purgeSequences
    reassignParticipants = imp62.reassignParticipants
    refreshObjects = imp22.refreshObjects
    refreshObjects2 = imp38.refreshObjects2
    removeChildren = imp0.removeChildren
    removeNamedReferenceFromDataset = imp27.removeNamedReferenceFromDataset
    removeParticipants = imp2.removeParticipants
    resetContextID = imp28.resetContextID
    revise = imp9.revise
    revise2 = imp2.revise2
    reviseObjects = imp30.reviseObjects
    saveAsNewItem = imp22.saveAsNewItem
    saveAsNewItem2 = imp2.saveAsNewItem2
    saveAsObjectAndRelate = imp65.saveAsObjectAndRelate
    saveAsObjects = imp53.saveAsObjects
    saveAsObjectsAndRelate = imp0.saveAsObjectsAndRelate
    setDisplayProperties = imp9.setDisplayProperties
    setLocalizedProperties = imp17.setLocalizedProperties
    setLocalizedPropertyValues = imp17.setLocalizedPropertyValues
    setOrRemoveImmunity = imp26.setOrRemoveImmunity
    setProperties = imp22.setProperties
    setProperties2 = imp23.setProperties
    setPropertiesAndDetectOverwrite = imp28.setPropertiesAndDetectOverwrite
    setTableProperties = imp44.setTableProperties
    unlinkAndDeleteObjects = imp70.unlinkAndDeleteObjects
    unloadObjects = imp71.unloadObjects
    validateAlternateIds = imp15.validateAlternateIds
    validateIdValue = imp7.validateIdValue
    validateItemIdsAndRevIds = imp26.validateItemIdsAndRevIds
    validateRevIds = imp53.validateRevIds
    validateValues = imp30.validateValues
    verifyExtension = imp23.verifyExtension
    whereReferenced = imp22.whereReferenced
    whereReferencedByRelationName = imp26.whereReferencedByRelationName
    whereUsed = imp22.whereUsed
    whereUsed2 = imp7.whereUsed


class ProjectLevelSecurityService(TcService):
    addOrRemoveProjectMembers = imp1.addOrRemoveProjectMembers
    assignOrRemoveObjects = imp4.assignOrRemoveObjects
    assignOrRemoveObjectsFromProjects = imp5.assignOrRemoveObjectsFromProjects
    changeOwningProgram = imp10.changeOwningProgram
    copyProjects = imp14.copyProjects
    copyProjects2 = imp5.copyProjects2
    createProjects = imp14.createProjects
    createProjects2 = imp5.createProjects2
    getDefaultProject = imp40.getDefaultProject
    getFirstLevelProjectTeamStructure = imp1.getFirstLevelProjectTeamStructure
    getModifiableProjects = imp1.getModifiableProjects
    getPrivilegeInProjects = imp1.getPrivilegeInProjects
    getProjectTeamChildNodes = imp1.getProjectTeamChildNodes
    getProjectTeams = imp14.getProjectTeams
    getProjectsForAssignOrRemove = imp5.getProjectsForAssignOrRemove
    getUserProjects = imp57.getUserProjects
    getUserProjects2 = imp10.getUserProjects2
    loadProjectDataForUser = imp59.loadProjectDataForUser
    modifyProjects = imp14.modifyProjects
    modifyProjects2 = imp5.modifyProjects2
    propagateData = imp61.propagateData
    setPropagationEnabledProperties = imp5.setPropagationEnabledProperties
    setUserPrivilege = imp1.setUserPrivilege


class DigitalSignatureService(TcService):
    applySignatures = imp3.applySignatures
    getSignatureMessages = imp3.getSignatureMessages
    voidSignatures = imp3.voidSignatures


class ReservationService(TcService):
    bulkCancelCheckout = imp6.bulkCancelCheckout
    bulkCheckin = imp6.bulkCheckin
    bulkCheckout = imp6.bulkCheckout
    cancelCheckout = imp8.cancelCheckout
    checkin = imp8.checkin
    checkout = imp8.checkout
    getReservationHistory = imp8.getReservationHistory
    okToCheckout = imp60.okToCheckout
    transferCheckout = imp69.transferCheckout


class FileManagementService(TcService):
    commitDatasetFiles = imp11.commitDatasetFiles
    commitDatasetFilesInBulk = imp12.commitDatasetFilesInBulk
    getDatasetWriteTickets = imp11.getDatasetWriteTickets
    getDigestInfoForDatasets = imp41.getDigestInfoForDatasets
    getDigestInfoForFiles = imp41.getDigestInfoForFiles
    getFileReadTickets = imp11.getFileReadTickets
    getTransientFileTicketsForUpload = imp55.getTransientFileTicketsForUpload
    replaceFiles = imp12.replaceFiles


class SessionService(TcService):
    connect = imp13.connect
    getAvailableServices = imp35.getAvailableServices
    getClientCacheData = imp36.getClientCacheData
    getDisplayStrings = imp42.getDisplayStrings
    getFavorites = imp13.getFavorites
    getGroupMembership = imp35.getGroupMembership
    getPreferences = imp35.getPreferences
    getPreferences2 = imp50.getPreferences
    getPreferences3 = imp51.getPreferences2
    getSessionGroupMember = imp35.getSessionGroupMember
    getShortcuts = imp51.getShortcuts
    getTCSessionInfo = imp50.getTCSessionInfo
    getTypeDescriptions = imp36.getTypeDescriptions
    getTypeDescriptions2 = imp56.getTypeDescriptions2
    licenseAdmin = imp58.licenseAdmin
    login = imp35.login
    login2 = imp42.login
    login3 = imp36.login
    loginSSO = imp35.loginSSO
    loginSSO2 = imp42.loginSSO
    loginSSO3 = imp36.loginSSO
    logout = imp35.logout
    refreshPOMCachePerRequest = imp63.refreshPOMCachePerRequest
    registerState = imp64.registerState
    setAndEvaluateIdDisplayRule = imp67.setAndEvaluateIdDisplayRule
    setFavorites = imp13.setFavorites
    setObjectPropertyPolicy = imp50.setObjectPropertyPolicy
    setObjectPropertyPolicy2 = imp42.setObjectPropertyPolicy
    setObjectPropertyPolicy3 = imp64.setObjectPropertyPolicy
    setPreferences = imp35.setPreferences
    setSessionGroupMember = imp35.setSessionGroupMember
    setUserSessionState = imp67.setUserSessionState
    setUserSessionStateAndUpdateDefaults = imp56.setUserSessionStateAndUpdateDefaults
    sponsoredLogin = imp56.sponsoredLogin
    sponsoredLoginSSO = imp56.sponsoredLoginSSO
    startOperation = imp68.startOperation
    stopOperation = imp68.stopOperation
    unregisterState = imp64.unregisterState
    updateObjectPropertyPolicy = imp36.updateObjectPropertyPolicy


class DispatcherManagementService(TcService):
    createDispatcherRequest = imp18.createDispatcherRequest


class StructureManagementService(TcService):
    createInStructureAssociations = imp20.createInStructureAssociations
    getPrimariesOfInStructureAssociation = imp20.getPrimariesOfInStructureAssociation
    getSecondariesOfInStructureAssociation = imp20.getSecondariesOfInStructureAssociation
    removeInStructureAssociations = imp20.removeInStructureAssociations


class ManagedRelationsService(TcService):
    createRelation = imp25.createRelation
    getManagedRelations = imp49.getManagedRelations
    getTraceReport = imp25.getTraceReport
    modifyRelation = imp25.modifyRelation


class LanguageInformationService(TcService):
    getAllTranslationStatuses = imp31.getAllTranslationStatuses
    getLanguagesList = imp31.getLanguagesList


class LOVService(TcService):
    getAttachedLOVs = imp32.getAttachedLOVs
    getInitialLOVValues = imp43.getInitialLOVValues
    getLOVAttachments = imp45.getLOVAttachments
    getNextLOVValues = imp43.getNextLOVValues
    validateLOVValueSelections = imp43.validateLOVValueSelections


class PropDescriptorService(TcService):
    getAttachedPropDescs = imp33.getAttachedPropDescs
    getAttachedPropDescs2 = imp34.getAttachedPropDescs2
    getCreateDesc = imp37.getCreateDesc


class OperationDescriptorService(TcService):
    getDeepCopyData = imp39.getDeepCopyData
    getSaveAsDesc = imp52.getSaveAsDesc


class LogicalObjectService(TcService):
    getLogicalObjects = imp46.getLogicalObjects
    getLogicalObjects2 = imp47.getLogicalObjects2
    getLogicalObjectsWithContext = imp48.getLogicalObjectsWithContext


class EnvelopeService(TcService):
    sendAndDeleteEnvelopes = imp66.sendAndDeleteEnvelopes



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, Group, Role, GroupMember
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetUserGroupMembersInputData(TcBaseObj):
    """
    This structure holds the input data to retrieve the group member information of a given user.
    
    :var user: The given user object.
    :var includeInactive: The option flag to include inactive group memers in returned group members data.
    :var includeUser: The option flag to include User objects in returned group member data.
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    """
    user: User = None
    includeInactive: bool = False
    includeUser: bool = False
    clientId: str = ''


@dataclass
class GetUserGroupMembersOutput(TcBaseObj):
    """
    This structure holds all user group member data for a given user.
    
    :var clientId: Unmodified value from the GetUserGroupMembersInputData.clientId. This can be used by the caller to
    indentify this data structure with the source input data.
    :var memebrs: All user group member data for a specific user.
    """
    clientId: str = ''
    memebrs: List[UserGroupMemberData] = ()


@dataclass
class GetUserGroupMembersResponse(TcBaseObj):
    """
    A list of user group members outputs, one for each of the given User object.
    
    :var outputs: List of group member information, one for each User object.
    :var serviceData: The object which holds the partial errors that occurred during retrieving user group member data.
    """
    outputs: List[GetUserGroupMembersOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GroupMemberInput(TcBaseObj):
    """
    This structure holds a GroupMember object, whose properties need to be updated and a map of property names and
    their corresponding values.
    
    :var groupMember: The GroupMember business object whose propeties  need to be updated.
    :var groupMemberPropValuesMap: A map of  property names and desired value(string/string).
    """
    groupMember: GroupMember = None
    groupMemberPropValuesMap: GroupMemberPropValuesMap = None


@dataclass
class RoleUser(TcBaseObj):
    """
    This structure holds role, user, group member status, default role flag and group admin privilege for a group
    member.
    
    :var role: The Role of a group member.
    :var user: The User of a group member.
    :var isActive: True if the group member is active.
    :var isDefaultRole: True if the this Role is the default in the group member.
    :var isGroupAdmin: True if the group member has administrative privilege
    """
    role: Role = None
    user: User = None
    isActive: bool = False
    isDefaultRole: bool = False
    isGroupAdmin: bool = False


@dataclass
class UserGroupMemberData(TcBaseObj):
    """
    This structure holds the all group member data under a group for the given user.
    
    :var group: The group of the group members in the RoleUser list.
    :var roleUsers: The list of RoleUser objects which belong to the same group.
    """
    group: Group = None
    roleUsers: List[RoleUser] = ()


"""
A map of property names and desired values (string/string).
"""
GroupMemberPropValuesMap = Dict[str, List[str]]
